-- A wrapper around our synchronous fifo that will format incoming avalon_ST_128 messages
-- into fifo transactions
-- Should be used a s a store / buffer inside my designs

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.avalon_st_pkg.all;

entity avalon_st_128_synchronous_fifo is
generic
(
  LOG2_DEPTH          : natural := 5;
  BACKPRESSURE_MARGIN : natural := 4
);
port
(
  clk : in  std_logic;
  rst : in  std_logic;

  input     : in  AVALON_ST_128;
  input_rdy : out std_logic;

  output     : out AVALON_ST_128;
  output_rdy : in  std_logic;

  fill : out std_logic_vector(31 downto 0);

  full  : out std_logic;
  empty : out std_logic
);
end entity;

architecture rtl of avalon_st_128_synchronous_fifo is

  constant FIFO_WIDTH : natural := input.data'length _ input.empty'length + 3; -- input.startofpacket'length + input.endofpacket'length + input.err'length (valid is not included here)
  constant FIFO_DEPTH : natural := 2**LOG2_DEPTH;

  signal fifo_write_enable : std_logic;
  signal fifo_write_data   : std_logic_vector(FIFO_WIDTH - 1 downto 0);

  signal fifo_read_enable : std_logic;
  signal fifo_read_data   : std_logic_vector(FIFO_WIDTH - 1 downto 0);

  signal fifo_internal_count : std_logic_vector(31 downto 0);

begin

  fifo_write_enable <= input.valid when rst /= '1' else '0'; -- No writes will be allowed upon a reset
  fifo_read_enable  <= output_rdy  when rst /= '1' else '0';

  basic_fifo : entity synchronous_fifo
  generic map
  (
    WIDTH => FIFO_WIDTH,
    DEPTH => FIFO_DEPTH
  )
  port map
  (
    clk => clk,
    rst => rst,

    write_enable => fifo_write_enable,
    write_data   => fifo_write_data,

    read_enable => fifo_read_enable,
    read_data   => fifo_read_data,

    empty        => empty,
    almost_empty => open,
    almost_full  => open,
    full         => full,

    count => fifo_internal_count
  );

  input_rdy <= '1' when (to_integer(unsigned(fifo_internal_count))) < (FIFO_DEPTH - BACKPRESSURE_MARGIN) else '0';
  fill      <= fifo_internal_count;

end architecture;