-- This block is a very basic implementation of a dual port ram
-- It operates on a single clock domain
-- This design was made for testing purposes
-- Currently no reset or cleaning functionality
-- Will always report back a read of the address specified

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity single_clock_dp_ram is
generic
(
  ADDRESS_WIDTH : natural := 16;
  DATA_WIDTH    : natural := 32
);
port
(
  clk : in  std_logic;

  -- A side
  a_address    : in  std_logic_vector(ADDRESS_WIDTH - 1 downto 0);
  a_write      : in  std_logic;
  a_write_data : in  std_logic_vector(DATA_WIDTH - 1    downto 0);
  a_read_data  : out std_logic_vector(DATA_WIDTH - 1    downto 0);

  -- B side
  b_address    : in  std_logic_vector(ADDRESS_WIDTH - 1 downto 0);
  b_write      : in  std_logic;
  b_write_data : in  std_logic_vector(DATA_WIDTH - 1    downto 0);
  b_read_data  : out std_logic_vector(DATA_WIDTH - 1    downto 0)
);
end entity;

architecture rtl of single_clock_dp_ram is

  type memory_type is array (2 ** ADDRESS_WIDTH - 1 downto 0) of std_logic_vector(DATA_WIDTH - 1 downto 0);
  signal internal_registers : memory_type := (others => (others => '0'));

begin

  dp_ram_proc : process(clk)
  begin
    if rising_edge(clk) then
      if a_write = '1' then
        internal_registers(to_integer(unsigned(a_address))) <= a_write_data;
      end if;

      a_read_data <= internal_registers(to_integer(unsigned(a_address)));

      if b_write = '1' then
        internal_registers(to_integer(unsigned(b_address))) <= b_write_data;
      end if;

      b_read_data <= internal_registers(to_integer(unsigned(b_address)));

    end if;
  end process;

end architecture;